@@ -53,6 +53,9 @@
 #include <QGestureEvent>
 #include <QPinchGesture>
 
+#include <QVBoxLayout>
+#include <QApplication>
+
 #include <algorithm>
 
 #include <cmath>
@@ -284,12 +287,12 @@ void qjackctlGraphPort::setPortTitle ( const QString& title )
 	static const int MAX_TITLE_LENGTH = 29;
 	static const QString ellipsis(3, '.');
 
-	QString text = m_title;
+	QString text = m_title.simplified();
 	const int nlength = text.indexOf(':');
 	if (nlength >= 0)
 		text.remove(0, nlength + 1);
 	if (text.length() >= MAX_TITLE_LENGTH + ellipsis.length())
-		text = ellipsis + text.right(MAX_TITLE_LENGTH);
+		text = ellipsis + text.right(MAX_TITLE_LENGTH).trimmed();
 
 	m_text->setPlainText(text);
 
@@ -742,9 +745,9 @@ void qjackctlGraphNode::setNodeTitle ( const QString& title )
 	static const int MAX_TITLE_LENGTH = 29;
 	static const QString ellipsis(3, '.');
 
-	QString text = m_title;
+	QString text = m_title.simplified();
 	if (text.length() >= MAX_TITLE_LENGTH  + ellipsis.length())
-		text = text.left(MAX_TITLE_LENGTH) + ellipsis;
+		text = text.left(MAX_TITLE_LENGTH).trimmed() + ellipsis;
 
 	m_text->setPlainText(text);
 }
@@ -1270,9 +1273,12 @@ qjackctlGraphCanvas::qjackctlGraphCanvas ( QWidget *parent )
 	QGraphicsView::setResizeAnchor(QGraphicsView::NoAnchor);
 	QGraphicsView::setDragMode(QGraphicsView::NoDrag);
 
-	const QPalette& pal = QGraphicsView::palette();
-	if (pal.base().color().value() > 192)
-		QGraphicsView::setBackgroundRole(QPalette::Mid);
+	QPalette pal = QGraphicsView::palette();
+	const QPalette::ColorRole role = QPalette::Window;
+	const QColor& color = pal.color(role);
+	pal.setColor(role, color.darker(120));
+	QGraphicsView::setPalette(pal);
+	QGraphicsView::setBackgroundRole(role);
 
 	m_editor = new QLineEdit(this);
 	m_editor->setFrame(false);
@@ -1570,13 +1576,20 @@ void qjackctlGraphCanvas::emitDisconnected (
 }
 
 
-// Rename notifiers.
+// Rename notifier.
 void qjackctlGraphCanvas::emitRenamed ( qjackctlGraphItem *item, const QString& name )
 {
 	emit renamed(item, name);
 }
 
 
+// Other generic notifier.
+void qjackctlGraphCanvas::emitChanged (void)
+{
+	emit changed();
+}
+
+
 // Item finder (internal).
 qjackctlGraphItem *qjackctlGraphCanvas::itemAt ( const QPointF& pos ) const
 {
@@ -1900,6 +1913,7 @@ void qjackctlGraphCanvas::mouseReleaseEvent ( QMouseEvent *event )
 			}
 			m_commands->push(
 				new qjackctlGraphMoveCommand(this, nodes, m_pos1, pos));
+			++nchanged;
 		}
 		// Close rubber-band lasso...
 		if (m_rubberband) {
@@ -2838,4 +2852,231 @@ QPointF qjackctlGraphCanvas::snapPos ( qreal x, qreal y ) const
 }
 
 
+//----------------------------------------------------------------------------
+// qjackctlGraphThumb::View -- Thumb graphics scene/view.
+
+class qjackctlGraphThumb::View : public QGraphicsView
+{
+public:
+
+	// Constructor.
+	View(qjackctlGraphCanvas *canvas)
+		: QGraphicsView(canvas->viewport()),
+			m_canvas(canvas), m_drag_state(DragNone)
+	{
+		QGraphicsView::setInteractive(false);
+
+		QGraphicsView::setRenderHints(QPainter::Antialiasing);
+		QGraphicsView::setRenderHint(QPainter::SmoothPixmapTransform);
+
+		QGraphicsView::setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+		QGraphicsView::setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+
+		QPalette pal = m_canvas->palette();
+		const QPalette::ColorRole role
+			= m_canvas->backgroundRole();
+		const QColor& color = pal.color(role);
+		pal.setColor(role, color.darker(120));
+		QGraphicsView::setPalette(pal);
+		QGraphicsView::setBackgroundRole(role);
+
+		QGraphicsView::setScene(m_canvas->scene());
+	}
+
+protected:
+
+	// Compute the view(port) rectangle.
+	QRect viewRect() const
+	{
+		const QRect& vrect
+			= m_canvas->viewport()->rect();
+		const QRectF srect(
+			m_canvas->mapToScene(vrect.topLeft()),
+			m_canvas->mapToScene(vrect.bottomRight()));
+		return QGraphicsView::viewport()->rect().intersected(QRect(
+			QGraphicsView::mapFromScene(srect.topLeft()),
+			QGraphicsView::mapFromScene(srect.bottomRight())))
+			.adjusted(0, 0, -1, -1);
+	}
+
+	// View paint method.
+	void paintEvent(QPaintEvent *event)
+	{
+		QGraphicsView::paintEvent(event);
+
+		QPainter painter(QGraphicsView::viewport());
+	//	const QPalette& pal = QGraphicsView::palette();
+	//	painter.setPen(pal.midlight().color());
+		const QRect& vrect
+			= QGraphicsView::viewport()->rect();
+		const QRect& vrect2 = viewRect();
+		const QColor shade(0, 0, 0, 64);
+		QRect rect;
+		// top shade...
+		rect.setTopLeft(vrect.topLeft());
+		rect.setBottomRight(QPoint(vrect.right(), vrect2.top() - 1));
+		if (rect.isValid())
+			painter.fillRect(rect, shade);
+		// left shade...
+		rect.setTopLeft(QPoint(vrect.left(), vrect2.top()));
+		rect.setBottomRight(vrect2.bottomLeft());
+		if (rect.isValid())
+			painter.fillRect(rect, shade);
+		// right shade...
+		rect.setTopLeft(vrect2.topRight());
+		rect.setBottomRight(QPoint(vrect.right(), vrect2.bottom()));
+		if (rect.isValid())
+			painter.fillRect(rect, shade);
+		// bottom shade...
+		rect.setTopLeft(QPoint(vrect.left(), vrect2.bottom() + 1));
+		rect.setBottomRight(vrect.bottomRight());
+		if (rect.isValid())
+			painter.fillRect(rect, shade);
+	}
+
+	// Handle mouse events.
+	//
+	void mousePressEvent(QMouseEvent *event)
+	{
+		QGraphicsView::mousePressEvent(event);
+
+		if (event->button() == Qt::LeftButton) {
+			m_drag_pos = event->pos();
+			m_drag_state = DragStart;
+		}
+	}
+
+	void mouseMoveEvent(QMouseEvent *event)
+	{
+		QGraphicsView::mouseMoveEvent(event);
+
+		if (m_drag_state == DragStart
+			&& (event->pos() - m_drag_pos).manhattanLength()
+				> QApplication::startDragDistance()) {
+			m_drag_state = DragMove;
+		}
+
+		if (m_drag_state == DragMove) {
+			m_canvas->centerOn(
+				QGraphicsView::mapToScene(event->pos()));
+		}
+	}
+
+	void mouseReleaseEvent(QMouseEvent *event)
+	{
+		QGraphicsView::mouseReleaseEvent(event);
+
+		if (m_drag_state != DragNone) {
+			m_canvas->centerOn(
+				QGraphicsView::mapToScene(event->pos()));
+			m_drag_state = DragNone;
+		}
+	}
+
+	void wheelEvent(QWheelEvent *) {} // Ignore wheel events.
+
+private:
+
+	// Instance members.
+	qjackctlGraphCanvas *m_canvas;
+
+	enum { DragNone = 0, DragStart, DragMove } m_drag_state;
+
+	QPoint m_drag_pos;
+};
+
+
+//----------------------------------------------------------------------------
+// qjackctlGraphThumb -- Thumb graphics scene/view.
+
+// Constructor.
+qjackctlGraphThumb::qjackctlGraphThumb ( qjackctlGraphCanvas *canvas, Position position )
+	: QFrame(canvas), m_canvas(canvas), m_position(position), m_view(nullptr)
+{
+	m_view = new View(m_canvas);
+
+	QVBoxLayout *layout = new QVBoxLayout();
+	layout->setSpacing(0);
+	layout->setContentsMargins(0, 0, 0, 0);
+	layout->addWidget(m_view);
+	QFrame::setLayout(layout);
+
+	QFrame::setFrameStyle(QFrame::Panel);
+	QFrame::setForegroundRole(QPalette::Window);
+
+	QObject::connect(m_canvas->horizontalScrollBar(),
+		SIGNAL(valueChanged(int)), SLOT(updateView()));
+	QObject::connect(m_canvas->verticalScrollBar(),
+		SIGNAL(valueChanged(int)), SLOT(updateView()));
+}
+
+
+// Destructor.
+qjackctlGraphThumb::~qjackctlGraphThumb (void)
+{
+}
+
+
+// Accessors.
+qjackctlGraphCanvas *qjackctlGraphThumb::canvas (void) const
+{
+	return m_canvas;
+}
+
+
+void qjackctlGraphThumb::setPosition ( Position position )
+{
+	m_position = position;
+
+	updatePosition();
+}
+
+
+qjackctlGraphThumb::Position qjackctlGraphThumb::position (void) const
+{
+	return m_position;
+}
+
+
+// Update view.
+void qjackctlGraphThumb::updatePosition (void)
+{
+	const QRect& rect = m_canvas->viewport()->rect();
+	const int w = rect.width()  / 4;
+	const int h = rect.height() / 4;
+	QFrame:setFixedSize(w + 1, h + 1); // 1px slack.
+
+	switch (m_position) {
+	case TopLeft:
+		QFrame::move(0, 0);
+		break;
+	case TopRight:
+		QFrame::move(rect.width() - w, 0);
+		break;
+	case BottomLeft:
+		QFrame::move(0, rect.height() - h);
+		break;
+	case BottomRight:
+		QFrame::move(rect.width() - w, rect.height() - h);
+		break;
+	case None:
+	default:
+		break;
+	}
+}
+
+
+// Update view.
+void qjackctlGraphThumb::updateView (void)
+{
+	updatePosition();
+
+	const qreal m = 24.0;
+	m_view->fitInView(
+		m_canvas->scene()->itemsBoundingRect()
+			.marginsAdded(QMarginsF(m, m, m, m)),
+		Qt::KeepAspectRatio);
+}
+
+
 // end of qjackctlGraph.cpp
